function x = maximize_likelihood(data, special_choices, number_of_types, mistake_size, Sanc, Sunanc, max_reps, starting_point)

	% setup matrix to capture the constraints on the parameters
		% all parameters are non-negative
			non_neg = eye(number_of_types-1+number_of_types*(Sanc+Sunanc+1)+number_of_types);
		% distribution over types must sum to 1
			non_neg(number_of_types-1+number_of_types*(Sanc+Sunanc+1)+number_of_types+1,1:number_of_types-1) = -1;
		% distribution over decision rules / solution concepts within type must sum to 1
			for types=1:number_of_types,
				non_neg(number_of_types-1+number_of_types*(Sanc+Sunanc+1)+number_of_types+1+types,number_of_types-1+(Sanc+Sunanc+1)*(types-1)+1:number_of_types-1+(Sanc+Sunanc+1)*(types-1)+(Sanc+Sunanc+1)) = -1;
			end;
		% probability of mistake must be less than or equal to 1
			for types=1:number_of_types,
				non_neg(number_of_types-1+number_of_types*(Sanc+Sunanc+1)+number_of_types+1+number_of_types+types,number_of_types-1+(Sanc+Sunanc+1)*number_of_types+types) = -1;
			end;
			
		% bounds
			bounds = [zeros(1,number_of_types-1+number_of_types*(Sanc+Sunanc+1)+number_of_types) ones(1,2*number_of_types+1)];

		% make constraints sparse
			non_neg = sparse(non_neg);
			bounds = sparse(bounds);

	% maximum likelihood for all games
		lik_fn = @(x) opt_mixture_lik_wrapper(data, x, special_choices, number_of_types, mistake_size, Sanc, Sunanc);
		opt = optimoptions('fmincon','display','none','StepTolerance', 10e-4, 'Algorithm', 'interior-point','SpecifyObjectiveGradient',true);
		likelihoods(1:max_reps) = 1000000;
	
	if size(starting_point,1) == 0,
		currentbest = 100000;
		z(1:max_reps,1:number_of_types-1+number_of_types*(Sanc+Sunanc+1)+number_of_types) = NaN;
		for rep=1:max_reps,
			start1 = ones(1,number_of_types)/number_of_types;
			start1 = start1(1:number_of_types-1);
			start2 = mnrnd(100, [1/(Sanc+1)*ones(1,Sanc) zeros(1,Sunanc) 1/(Sanc+1) 0], number_of_types)'/100;
			start2 = reshape(start2(1:Sanc+Sunanc+1,:),1,number_of_types*(Sanc+Sunanc+1));
			start3 = mnrnd(100, [.05 .95], number_of_types)'/100;
			start3 = reshape(start3(1,:),1,number_of_types);
			z(rep,:) = fmincon(lik_fn, [start1 start2 start3], -non_neg, bounds, [], [], [], [], [], opt);
			likelihoods(rep) = lik_fn(z(rep,:));
			if likelihoods(rep) < currentbest,
				x = z(rep,:);
				currentbest = lik_fn(z(rep,:));
			end;
			fprintf(1, '.');
			convertToEstResults(z(rep,:), number_of_types, [], Sanc, Sunanc)
		end;
	end;
	
	if size(starting_point,1) > 0,
		x = fmincon(lik_fn, starting_point, -non_neg, bounds, [], [], [], [], [], opt);
	end;

end