function [likelihood, grad, likelihood_of_i_from_type] = mixture_lik(data, dist_over_types, prob_of_mistakes, dist_over_solns, special_choices, number_of_types, mistake_size, Sanc, Sunanc)

	N = size(data,1);
	G = size(data,2);
	
	likelihood = 0;
	likelihood_of_i(1:N) = 0;
	likelihood_of_i_from_type(1:N,1:number_of_types) = 0;
	likelihood_of_i_from_type_unadj(1:N,1:number_of_types) = 0;
	likelihood_of_i_from_type_in_game(1:N,1:number_of_types,1:G) = 0;
	likelihood_of_i_from_type_in_game_with_soln(1:N,1:number_of_types,1:G,1:Sanc+Sunanc+2) = 0;
	mistakeL(1:Sanc+1,1:G) = 0;
	mistakeU(1:Sanc+1,1:G) = 0;
	mistake_size_in_game(1:G) = 0;
	mistake_multiply(1:N,1:G,1:Sanc+Sunanc+2) = 0;
	mistakes_multiply_of_i_from_type_in_game(1:N,1:number_of_types,1:G) = 0;
	mistakes_multiply_of_i_from_type(1:N,1:number_of_types) = 0;
	
	for g=1:G,
		mistake_size_in_game(g) = mistake_size*(special_choices{1}(2,g)-special_choices{1}(1,g))/200;

		if mistake_size_in_game(g) > 0,
			for s=1:Sanc,
				mistakeL(s,g) = max(special_choices{1}(1,g), special_choices{2}(s,g) - mistake_size_in_game(g));
				mistakeU(s,g) = min(special_choices{1}(2,g), special_choices{2}(s,g) + mistake_size_in_game(g));
			end;
			mistakeL(Sanc+1,g) = max(special_choices{1}(1,g), special_choices{3}(g) - mistake_size_in_game(g));
			mistakeU(Sanc+1,g) = min(special_choices{1}(2,g), special_choices{3}(g) + mistake_size_in_game(g));
		else,
			for s=1:Sanc+1,
				mistakeL(s,g) = 1;
				mistakeU(s,g) = 0; 
			end;			
		end;
	end;
	
	for i = 1:N,
	
		for g=1:G,
							
			% unanchored reasoning - 0 steps
			
				likelihood_of_i_from_type_in_game_with_soln(i,1:number_of_types,g,Sanc+Sunanc+1) = (special_choices{1}(1,g) <= data(i,g))*(data(i,g) <= special_choices{1}(2,g))*1/(special_choices{1}(2,g)-special_choices{1}(1,g));
			
			% unanchored reasoning - more than 0 steps

				for s=1:Sunanc,
					if special_choices{4}(2,s,g) > special_choices{4}(1,s,g),
						likelihood_of_i_from_type_in_game_with_soln(i,1:number_of_types,g,Sanc+s) = (special_choices{4}(1,s,g) <= data(i,g))*(data(i,g) <= special_choices{4}(2,s,g))*1/(special_choices{4}(2,s,g)-special_choices{4}(1,s,g));
					else,
						likelihood_of_i_from_type_in_game_with_soln(i,1:number_of_types,g,Sanc+s) = (special_choices{4}(1,s,g) <= data(i,g))*(data(i,g) <= special_choices{4}(2,s,g));
					end;
				end;
				
			% mistake multiplier for anchored reasoning and Nash 
			% --- subtract indicator for action that arises without computational mistake, to account for the (1-delta) probability in light of including same indicator below in likelihood_of_i_from_type_in_game_with_soln
			
				for s=1:Sanc,	
					mistake_multiply(i,g,s) = (mistakeL(s,g) <= data(i,g))*(data(i,g) <= mistakeU(s,g))*1/(mistakeU(s,g)-mistakeL(s,g))-(data(i,g) == special_choices{2}(s,g));
				end;
				
				mistake_multiply(i,g,Sanc+Sunanc+2) = (mistakeL(Sanc+1,g) <= data(i,g))*(data(i,g) <= mistakeU(Sanc+1,g))*1/(mistakeU(Sanc+1,g)-mistakeL(Sanc+1,g))-(data(i,g) == special_choices{3}(g));
			
			for type_index = 1:number_of_types,
			
				% anchored reasoning
					for s=1:Sanc,
						likelihood_of_i_from_type_in_game_with_soln(i,type_index,g,s) = (data(i,g) == special_choices{2}(s,g)) + prob_of_mistakes(type_index)*mistake_multiply(i,g,s);
					end;
				
				% Nash equilibrium
					likelihood_of_i_from_type_in_game_with_soln(i,type_index,g,Sanc+Sunanc+2) = (data(i,g) == special_choices{3}(g)) + prob_of_mistakes(type_index)*mistake_multiply(i,g,Sanc+Sunanc+2);
					
			end;
				
		end;
			
	end;
	
	for i=1:N,
		for type_index=1:number_of_types,
			for g=1:G,
				likelihood_of_i_from_type_in_game(i,type_index,g) = squeeze(likelihood_of_i_from_type_in_game_with_soln(i,type_index,g,:))'*dist_over_solns(type_index,:)';
				
				mistakes_multiply_of_i_from_type_in_game(i,type_index,g) = squeeze(mistake_multiply(i,g,:))'*dist_over_solns(type_index,:)';
			end;
			
			% adjust likelihood of i from type by baseline probability of type	
				likelihood_of_i_from_type(i,type_index) = dist_over_types(type_index)*prod(likelihood_of_i_from_type_in_game(i,type_index,:));	
				
				likelihood_of_i_from_type_unadj(i,type_index) = prod(likelihood_of_i_from_type_in_game(i,type_index,:));
				
				mistakes_multiply_of_i_from_type(i,type_index) = dist_over_types(type_index)*prod(mistakes_multiply_of_i_from_type_in_game(i,type_index,:));
		end;
	end;
	
	likelihood_of_i = sum(likelihood_of_i_from_type, 2);
	likelihood = sum(log(likelihood_of_i(:)));
	
	% derivatives with respect to dist_over_types
		grad_disttypes(1:number_of_types) = 0;

		for type_index=1:number_of_types,
			grad_disttypes(type_index) = (1./likelihood_of_i)'*likelihood_of_i_from_type_unadj(:,type_index);
		end;

	% derivatives with respect to prob_of_mistakes
		grad_of_i_probmist(1:N,1:number_of_types) = 0;
		grad_probmist(1:number_of_types) = 0;
		
		allotherindices = cell(1,G);
		for g=1:G,	
			allotherindices{g} = setdiff(1:G, g);
		end;

		for type_index=1:number_of_types,
			for i=1:N,
				for g=1:G,
					grad_of_i_probmist(i, type_index) = grad_of_i_probmist(i, type_index) + mistakes_multiply_of_i_from_type_in_game(i,type_index,g)*prod(likelihood_of_i_from_type_in_game(i,type_index,allotherindices{g}));
				end;
			end;

			grad_of_i_probmist(i, type_index) = grad_of_i_probmist(i, type_index)*dist_over_types(type_index);
			grad_probmist(type_index) = (1./likelihood_of_i)'*grad_of_i_probmist(:,type_index);
		end;

	% derivatives with respect to dist_over_solns
		grad_of_i_distsolns(1:N,1:number_of_types,1:Sanc+Sunanc+2) = 0;
		grad_distsolns(1:number_of_types,1:Sanc+Sunanc+2) = 0;

		for type_index=1:number_of_types,
			for s=1:Sanc+Sunanc+2,
				for i=1:N,
					for g=1:G,						
						grad_of_i_distsolns(i,type_index,s) = grad_of_i_distsolns(i,type_index,s) + likelihood_of_i_from_type_in_game_with_soln(i,type_index,g,s)*prod(likelihood_of_i_from_type_in_game(i,type_index,allotherindices{g}));
					end;
					grad_of_i_distsolns(i,type_index,s) = grad_of_i_distsolns(i,type_index,s)*dist_over_types(type_index);
				end;
				grad_distsolns(type_index,s) = (1./likelihood_of_i)'*grad_of_i_distsolns(:,type_index,s);
			end;
		end;

	% gradient -- adjusted for the parameter summation constraints
		grad = [grad_disttypes(1:number_of_types-1)-grad_disttypes(number_of_types) reshape((grad_distsolns(:,1:Sanc+Sunanc+1)-repmat(grad_distsolns(:,Sanc+Sunanc+2),1,Sanc+Sunanc+1))',1,number_of_types*(Sanc+Sunanc+1)) grad_probmist];
end