% Replication file for "An empirical model of non-equilibrium behavior in games" by Brendan Kline

clear variables
close all

% Set estimation parameters
	% set computeStdErrors = 1, and range_est_types to 5:5 and finaltypes = 5 when final estimates
	% set computeStdErrors = 0, and range_est_types to 1:7 and finaltypes = 5 when model selection
	use_generated_data = 0;
	computeStdErrors = 1;
	total_subsamples = 100;
	
	range_est_types = [5:5];	
	final_types = 5;
	
	% size of mistakes: either 2.5 or 0
		mistake_size = 0;
		mistakes_free_parms_adjustment = (mistake_size > 0);
		
	% set of decision rules / solution concepts used
		Sanc = 2;
		Sunanc = 1;

% load the data --- convert SubjectsGuesses.xls from original CGC paper to comma separated format, named subjects_CGC.csv
	if use_generated_data == 0,
		M = load('subjects_CGC.csv');
	elseif use_generated_data == 1,
		% reset estimation parameters consistent with the generated data
		M = load('generated.csv');
		computeStdErrors = 1;
		range_est_types = [2:2];
		final_types = 2;
		mistake_size = 2.5;
		mistakes_free_parms_adjustment = (mistake_size > 0);
	end;
	RandStream.setGlobalStream(RandStream('mt19937ar','Seed', 1));
	N = size(M,1);
	G = 16;
	graph_adjust = 300;
	alpha = 0.025;
	
% parameters of the games in the same order as in the data
	own_ps = [0.7 0.5 1.5 1.3 0.5 0.7 0.7 0.7 0.5 0.7 1.3 1.3 1.5 1.3 1.5 1.5];
	mins = [100 100 300 300 100 300 300 100 100 300 300 300 100 100 100 100];
	maxs = [500 900 500 900 900 900 500 500 900 500 900 900 500 900 900 500];
	anc_guess{1} = [350 150 200 350 500 520 780 780 150 500 350 780 350 600 210 450];
	anc_guess{2} = [105 175 175 300 500 650 900 900 250 225 546 455 420 525 315 315];
	anc_guess{3} = [122.5 100 150 300 500 650 900 900 112.5 375 318.5 709.8 367.5 630 220.5 472.5];
	Nashguess = [100 100 150 300 500 650 900 900 100 150 300 390 500 750 350 500];
	
	unanc_lower{1} = [100 100 150 300 450 390 390 390 100 150 300 390 300 450 100 150];
	unanc_upper{1} = [500 250 250 500 500 650 900 900 250 500 630 900 500 750 350 500];	
	unanc_lower{2} = [100 100 150 300 500 585 507 507 100 150 300 390 315 450 105 150];
	unanc_upper{2} = [175 250 250 300 500 650 900 900 250 375 630 819 500 750 350 500];
	unanc_lower{3} = [100 100 150 300 500 650 659.1 659.1 100 150 300 390 315 472.5 105 157.5];
	unanc_upper{3} = [175 100 150 300 500 650 900 900 187.5 375 573.3 819 500 750 350 500];
	
	companiongame = @(g) g - 1 + 2*mod(g,2);
	
% rearrange games to be in the same order as CGC
		rearrange = [1 9 2 10 3 11 4 12 5 13 6 14 7 15 8 16];

		own_ps = own_ps(rearrange);
		mins = mins(rearrange);
		maxs = maxs(rearrange);
		
	if use_generated_data == 0,
		M = M(:,[1 rearrange+1]);	
	end;
	M = M(:,2:17);

% table of experiments
	for g=1:16,
		gp = companiongame(g);
		fprintf(1, '%3.0f & %3.0f & %3.0f & %3.0f & %3.0f & %3.2f & %3.2f', g, mins(g), maxs(g), mins(gp), maxs(gp), own_ps(g), own_ps(gp));
		for s=1:Sanc,
			fprintf(1, ' & %3.0f', anc_guess{s}(g));
		end;
		for s=1:Sunanc,
			fprintf(1, ' & [%3.0f, %3.0f]', unanc_lower{s}(g), unanc_upper{s}(g));
		end;	
		fprintf(1, ' & %3.0f \\\\ \n', Nashguess(g));
	end;
	fprintf(1, '\n \n \n');
	
% table of experiments -- based on computed decision rules / solution concepts
	
	truncate_to_actionspace = @(x) x(1)*(x(1) > x(2)) + x(2)*(x(1) <= x(2))*(x(2) <= x(3)) + x(3)*(x(2) > x(3));
	for g=1:16,
		gp = companiongame(g);
		anc_guess_own{1}(g) = truncate_to_actionspace([mins(g) own_ps(g)*(mins(gp)+maxs(gp))/2 maxs(g)]);
	end;
	for s=2:3,
		for g=1:16,
			gp = companiongame(g);
			anc_guess_own{s}(g) = truncate_to_actionspace([mins(g) own_ps(g)*anc_guess_own{s-1}(gp) maxs(g)]);
		end;
	end;
	% verify Nashes are mutually best responses
	for g=1:16,
		gp = companiongame(g);
		Nashguess_own(g) = truncate_to_actionspace([mins(g) own_ps(g)*Nashguess(gp) maxs(g)]);
	end;
	for g=1:16,
		gp = companiongame(g);
		unanc_lower_own{1}(g) = truncate_to_actionspace([mins(g) own_ps(g)*mins(gp) maxs(g)]);
		unanc_upper_own{1}(g) = truncate_to_actionspace([mins(g) own_ps(g)*maxs(gp) maxs(g)]);
	end;
	for s=2:3,
		for g=1:16,
			gp = companiongame(g);
			unanc_lower_own{s}(g) = truncate_to_actionspace([mins(g) own_ps(g)*unanc_lower_own{s-1}(gp) maxs(g)]);
			unanc_upper_own{s}(g) = truncate_to_actionspace([mins(g) own_ps(g)*unanc_upper_own{s-1}(gp) maxs(g)]);
		end;
	end;
	for g=1:16,
		gp = companiongame(g);
		fprintf(1, 'Game %3.0f \n', g);
		fprintf(1, '%3.0f & %3.0f & %3.0f & %3.0f & %3.0f & %3.2f & %3.2f & %3.2f & %3.2f & [%3.2f, %3.2f] & [%3.2f, %3.2f] & [%3.2f, %3.2f] & %3.2f \\\\ \n', g, mins(g), maxs(g), mins(gp), maxs(gp), own_ps(g), own_ps(gp), anc_guess{1}(g), anc_guess{2}(g), unanc_lower{1}(g), unanc_upper{1}(g), unanc_lower{2}(g), unanc_upper{2}(g), unanc_lower{3}(g), unanc_upper{3}(g), Nashguess(g));
		fprintf(1, '%3.0f & %3.0f & %3.0f & %3.0f & %3.0f & %3.2f & %3.2f & %3.2f & %3.2f & [%3.2f, %3.2f] & [%3.2f, %3.2f] & [%3.2f, %3.2f] & %3.2f \\\\ \n', g, mins(g), maxs(g), mins(gp), maxs(gp), own_ps(g), own_ps(gp), anc_guess_own{1}(g), anc_guess_own{2}(g), unanc_lower_own{1}(g), unanc_upper_own{1}(g), unanc_lower_own{2}(g), unanc_upper_own{2}(g), unanc_lower_own{3}(g), unanc_upper_own{3}(g), Nashguess_own(g));
		fprintf(1, '%3.0f & %3.0f & %3.0f & %3.0f & %3.0f & %3.2f & %3.2f & %3.2f & %3.2f & [%3.2f, %3.2f] & [%3.2f, %3.2f] & [%3.2f, %3.2f] & %3.2f \\\\ \n', g, mins(g), maxs(g), mins(gp), maxs(gp), own_ps(g), own_ps(gp), anc_guess{1}(g)-anc_guess_own{1}(g), anc_guess{2}(g)-anc_guess_own{2}(g), unanc_lower{1}(g)-unanc_lower_own{1}(g), unanc_upper{1}(g)-unanc_upper_own{1}(g), unanc_lower{2}(g)-unanc_lower_own{2}(g), unanc_upper{2}(g)-unanc_upper_own{2}(g), unanc_lower{3}(g)-unanc_lower_own{3}(g), unanc_upper{3}(g)-unanc_upper_own{3}(g), Nashguess(g)-Nashguess_own(g));
		fprintf(1, '\n');
	end;
	
% plot the raw data
	for g=1:16,
		figure;
		plot_cdf(M(:,g),[0 0 1], 0.5);
		hold on;
		
		anc_start{1} = 0;
		anc_start{2} = 0;
		anc_start{3} = 0;
		startNash = 0;
		if anc_guess{1}(g) == anc_guess{2}(g), 
			anc_start{2} = anc_start{1} + 0.1;
		end;
		if anc_guess{2}(g) == anc_guess{3}(g),
			anc_start{3} = anc_start{2} + 0.1;
		end;
		if anc_guess{3}(g) == Nashguess(g),
			startNash = anc_start{3} + 0.1;
		end;
		
		unanc_start{1} = 1;
		unanc_start{2} = 1;
		unanc_start{3} = 1;
		if unanc_lower{1}(g) == unanc_lower{2}(g) || unanc_upper{1}(g) == unanc_upper{2}(g),
			unanc_start{2} = unanc_start{1} - 0.1;
		end;
		if unanc_lower{2}(g) == unanc_lower{3}(g) || unanc_upper{2}(g) == unanc_upper{3}(g),
			unanc_start{3} = unanc_start{2} - 0.1;
		end;
		
		for s=1:3,
			line([anc_guess{s}(g) anc_guess{s}(g)],[anc_start{s}+0 anc_start{s}+0.1],'Color','m','LineStyle','--');
			text(anc_guess{s}(g)-0.2/graph_adjust,anc_start{s}+0.01,int2str(s),'Rotation',90,'Color','m');
		end;
		line([Nashguess(g) Nashguess(g)],[startNash+0 startNash+0.1],'Color','m','LineStyle','--');
		text(Nashguess(g)-0.2/graph_adjust,startNash+0.01,'Nash','Rotation',90,'Color','m');
		
		for s=1:3,
			line([unanc_lower{s}(g) unanc_lower{s}(g)],[unanc_start{s}+0 unanc_start{s}-0.1],'Color','r','LineStyle','--');
			text(unanc_lower{s}(g)-0.2/graph_adjust,unanc_start{s}-0.05,int2str(s),'Rotation',90,'Color','r');
			line([unanc_upper{s}(g) unanc_upper{s}(g)],[unanc_start{s}+0 unanc_start{s}-0.1],'Color','r','LineStyle','--');
			text(unanc_upper{s}(g)-0.2/graph_adjust,unanc_start{s}-0.05,int2str(s),'Rotation',90,'Color','r');
		end;

		title(strcat('Game', 32, num2str(g)),'FontWeight','bold')
		axis([mins(g) maxs(g) 0 1])
		set(gca, 'LooseInset', get(gca, 'TightInset'));
		set(gcf, 'PaperPosition', [0 0 5 5]);
		set(gcf, 'PaperSize', [5 5]);
		if use_generated_data == 0,
			print('-dpdf','-r0', strcat('figures/allactions', num2str(g), '.pdf'))
		end;
	end;	
	
% model estimation
	
	clear infocrit loglik overall est_results x x_sub est_results_sub subsample_draws subsample_draws_recentered confidenceIntLB confidenceIntUB
	
	x = cell(max(range_est_types),1);
	loglik(range_est_types) = NaN;
	est_results = cell(max(range_est_types),1);
	overall = cell(max(range_est_types),1);
	x_sub = cell(max(range_est_types),total_subsamples);
	est_results_sub = cell(max(range_est_types),total_subsamples);
	subsample_draws = cell(max(range_est_types),1);
	subsample_draws_recentered = cell(max(range_est_types),max(range_est_types),Sanc+Sunanc+4);
	confidenceIntLB = cell(max(range_est_types),1);
	confidenceIntUB = cell(max(range_est_types),1);	
	
	for g=1:G,
		special_choices{1}(1,g) = mins(g);
		special_choices{1}(2,g) = maxs(g);
		special_choices{2}(:,g) = [anc_guess{1}(g) anc_guess{2}(g) anc_guess{3}(g)]';
		special_choices{3}(g) = Nashguess(g);
		special_choices{4}(1,:,g) = [unanc_lower{1}(g) unanc_lower{2}(g) unanc_lower{3}(g)]';
		special_choices{4}(2,:,g) = [unanc_upper{1}(g) unanc_upper{2}(g) unanc_upper{3}(g)]';
	end;
	
	infocrit(1:8,1:3) = Inf;
	
	% number of free parameters, R = x(1) and Sanc = x(2) and Sunanc = x(3) ... +1 from Nash +1 from level 0 (+ 1 from mistake probability, possibly) -1 from adding up
		freeparms_fn = @(x) (x(1)-1)+x(1)*(x(2)+x(3)+1+mistakes_free_parms_adjustment);
	
	for number_of_types = range_est_types,
		RandStream.setGlobalStream(RandStream('mt19937ar','Seed', 1));

		% maximize the likelihood
			fprintf(1, 'Computing maximum likelihood estimates for %3.0f types... \n', number_of_types);
			lik_fn = @(x) mixture_lik_wrapper(M, x, special_choices, number_of_types, mistake_size, Sanc, Sunanc);
			x{number_of_types} = maximize_likelihood(M, special_choices, number_of_types, mistake_size, Sanc, Sunanc, 5, []);

		% evaluate fit by likelihood criterion
			freeparms = freeparms_fn([number_of_types Sanc Sunanc]);
			loglik(number_of_types) = lik_fn(x{number_of_types})
			infocrit(number_of_types,1) = -2*lik_fn(x{number_of_types}) + (freeparms)*log(N)
			infocrit(number_of_types,2) = -2*lik_fn(x{number_of_types}) + 2*(freeparms);
			infocrit(number_of_types,3) = infocrit(number_of_types,2) + 2*(freeparms)*(freeparms+1)/(N-freeparms-1)

		% rearrange the estimates to be legible
			[est_results{number_of_types}, overall{number_of_types}] = convertToEstResults(x{number_of_types}, number_of_types, [], Sanc, Sunanc)

			for types=1:number_of_types
				for parm = 1:Sanc+Sunanc+4,
					fprintf(1, '%3.2f', est_results{number_of_types}(types,parm));
					if parm < Sanc+Sunanc+4,
						fprintf(1, ' & ');
					end;
				end;
				fprintf(1, '\\\\ \n');
			end;
			
		% compute standard errors by subsampling
			if computeStdErrors,
				RandStream.setGlobalStream(RandStream('mt19937ar','Seed', 1));
				ssample_size = floor(2*N/3);
				x_sub{number_of_types,total_subsamples} = NaN*ones(1,size(x{number_of_types},2));
				fprintf(1, 'Computing standard errors by subsampling... \n');
				for subsamples=1:total_subsamples,
					sample_indicators = datasample(1:N,ssample_size,'Replace',false);
					M_sub = M(sample_indicators,:);
					x_sub{number_of_types,subsamples} = maximize_likelihood(M_sub, special_choices, number_of_types, mistake_size, Sanc, Sunanc, 1, x{number_of_types});
					fprintf(1, '.');
					if mod(subsamples,10) == 0,
						fprintf(1, '%3.0f', subsamples);
						if mod(subsamples,50) == 0,
							fprintf(1, '\n');
						end;
					end;
				end;
				fprintf(1, '\n');
				
				est_results_sub{number_of_types,total_subsamples} = NaN*ones(1,size(est_results{number_of_types},2));
				for subsamples=1:total_subsamples,
					est_results_sub{number_of_types,subsamples} = convertToEstResults(x_sub{number_of_types,subsamples}, number_of_types, est_results{number_of_types}, Sanc, Sunanc);
				end;
				
				subsample_draws{number_of_types}(1:total_subsamples,1:number_of_types,1:Sanc+Sunanc+4) = NaN;
				subsample_draws_recentered{number_of_types,number_of_types,Sanc+Sunanc+4} = NaN;
				confidenceIntLB{number_of_types}(1:number_of_types,1:Sanc+Sunanc+4) = NaN;
				confidenceIntUB{number_of_types}(1:number_of_types,1:Sanc+Sunanc+4) = NaN;
				for types=1:number_of_types,
					for parameter=1:Sanc+Sunanc+4,
						for subsamples=1:total_subsamples,
							subsample_draws{number_of_types}(subsamples,types,parameter) = est_results_sub{number_of_types,subsamples}(types,parameter);
						end;
						subsample_draws{number_of_types}(:,types,parameter) = sort(subsample_draws{number_of_types}(:,types,parameter));
						subsample_draws_recentered{number_of_types,types,parameter} = sqrt(ssample_size)*(subsample_draws{number_of_types}(:,types,parameter) - est_results{number_of_types}(types,parameter));
						
						confidenceIntLB{number_of_types}(types,parameter) =  est_results{number_of_types}(types,parameter) - (sqrt(N))^(-1)*subsample_draws_recentered{number_of_types,types,parameter}(floor(total_subsamples*(1-alpha)));
						confidenceIntUB{number_of_types}(types,parameter) =  est_results{number_of_types}(types,parameter) - (sqrt(N))^(-1)*subsample_draws_recentered{number_of_types,types,parameter}(ceil(total_subsamples*alpha));
					end;
				end;
			end;	
	end;
	clear subsample_draws M_sub subsamples g types parameter
		
	% print table of info criteria (BIC and AICc)
		for types=1:8,
			fprintf(1, '%3.0f & %3.2f & %3.2f & %3.2f & %3.2f \\\\ \n', types, infocrit(types,1), infocrit(types,1) - min(infocrit(:,1)), infocrit(types,3), infocrit(types,3) - min(infocrit(:,3)));
		end;
	
	% print table of estimation results
		if computeStdErrors,
			truncate_to_01 = @(x) 1*(x >= 1) + 0*(x <= 0) + x*(x < 1)*(x > 0);
			confidenceIntLB{final_types} = arrayfun(truncate_to_01,confidenceIntLB{final_types});
			confidenceIntUB{final_types} = arrayfun(truncate_to_01,confidenceIntUB{final_types});
		end;
		
		fprintf(1, '                    ');
		for parm=1:Sanc,
			fprintf(1, '& %2.0fan ', parm);			
		end;
		fprintf(1, '& 0un ');
		for parm=1:Sunanc,
			fprintf(1, '& %2.0fun ', parm);		
		end;
		fprintf(1, '& Nash & Pr(type) & Pr(mistake) \\\\ \n');
		for types=1:final_types,
			fprintf(1, '\\multirow{2}{*}{%3.0f} ', types);
			for parm=1:Sanc+Sunanc+4,
				fprintf(1, ' & %3.2f', est_results{final_types}(types,parm));
			end;
			fprintf(1, ' \\\\ \n');
			
			if computeStdErrors,
				for parm=1:Sanc+Sunanc+4,
					fprintf(1, '& (%3.2f, %3.2f)', confidenceIntLB{final_types}(types,parm), confidenceIntUB{final_types}(types,parm));
				end;
				fprintf(1, '\\\\ \n \\hline \n');
			else,
				for parm=1:Sanc+Sunanc+4,
					fprintf(1, '& ');
				end;
				fprintf(1, '\\\\ \n \\hline \n');				
			end;
		end;
	
	if use_generated_data == 0,
		zip('figures',{'figures/*.pdf'});
	end;